import type { ResponseInfo } from '@/types/axios'
import type { TableReq } from '@/types/Tables/MenuTable/menuTableReq'
import type { PaginationConfig } from '@/types/Tables/pagination'
import {
  type BaseFieldInfo,
  type BaseFieldItem,
  type TableData,
  type TableFields,
  TableFieldType,
} from '@/types/Tables/table'
import type { BaseTableInfo } from '@/types/Tables/tablePageData'

import axiosInstance from '@/utils/axios/axiosInstance'

import { resetReactive } from '@/utils/common'
import type { Ref } from 'vue'
import { utils, writeFileXLSX } from 'xlsx'

export function useTable() {
  let controller: AbortController | null = null // 请求控制器，用于取消请求
  let tableReqList: Array<string> = [] // 表格的请求列表，暂时只存了url，可以存其他的

  /**
   * @description: 获取表格数据
   * @param reqInfo 请求信息
   */
  const getData = async (reqInfo: TableReq): Promise<Array<TableData>> => {
    try {
      let { url, config } = reqInfo
      tableReqList.push(url) // 添加请求
      controller && controller.abort() // 取消上一次请求
      controller = new AbortController()

      config.signal = controller.signal
      const res = (await axiosInstance.get(url, config)) as ResponseInfo

      if (res.code !== 0) throw new Error('请求失败')
      return res.data as TableData[]
    } catch (err) {
      console.log('数据请求失败')
      console.log(err)
      return []
    } finally {
      tableReqList.pop() // 请求完成后移除一个请求
    }
  }

  /**
   * @description: 判断当前字段是否是需要固定的字段
   * @param  fixedFields 固定字段信息
   * @param info 字段信息
   * @return 是否需要固定
   */
  const isFixedField = (
    fixedFields: Array<string> | undefined,
    info: BaseFieldInfo,
  ): boolean => {
    return !!fixedFields && fixedFields.includes(info.name)
  }

  /**
   * @description: 更新自定义指标和表格字段
   * @param  tableInfo 表格信息
   * @param  activeName 当前激活的导航
   * @param  newTableFiles  新的表格字段信息
   * @param  newSortedTableFields 新的表格字段信息
   * @return 更新是否成功
   */
  const updateTableFields = (
    tableInfo: BaseTableInfo,
    activeName: string,
    newTableFiles: Array<BaseFieldItem<TableFields>>,
    newSortedTableFields: Array<TableFields>,
  ): Promise<boolean> => {
    return new Promise((resolve, reject) => {
      try {
        let oldTableFields = tableInfo[activeName].fields
        let oldSortedFields = tableInfo[activeName].tableSortedFields

        oldTableFields.splice(0, oldTableFields.length, ...newTableFiles)

        oldSortedFields.splice(
          0,
          oldSortedFields.length,
          ...newSortedTableFields,
        )
        resolve(true)
      } catch (err) {
        console.log('更新失败')
        console.log(err)
        reject(false)
      }
    })
  }

  /**
   * @description: 初始化自定义指标的字段信息
   * @param indicatorFields 自定义指标字段信息
   * @param propsTableFields 表格字段信息
   * @param defaultActiveNav 默认激活的导航
   */
  const initIndicatorFields = (
    indicatorFields: Array<BaseFieldItem<TableFields>>,
    propsTableFields: Array<BaseFieldItem<BaseFieldInfo>>,
    defaultActiveNav: Ref<string>,
  ) => {
    resetReactive(indicatorFields)
    indicatorFields.splice(
      0,
      indicatorFields.length,
      ...JSON.parse(JSON.stringify(propsTableFields)),
    )
    defaultActiveNav.value = indicatorFields[0].name
  }

  /**
   * @description: 初始化表格字段，把所有字段都放到一个数组中
   * @param tableFieldsInfo 表格字段信息
   * @param sortedTableFields 排序过的表格字段信息
   * @param indicatorFields 自定义指标字段信息
   */
  const initTableFields = (
    tableFieldsInfo: Array<BaseFieldInfo>,
    sortedTableFields: Array<BaseFieldInfo> | undefined,
    indicatorFields: Array<BaseFieldItem<TableFields>>,
  ) => {
    resetReactive(tableFieldsInfo)
    // 如果传入了排序字段，则直接使用排序字段，说明已经有缓存了
    if (sortedTableFields && sortedTableFields.length > 0) {
      tableFieldsInfo.splice(0, tableFieldsInfo.length, ...sortedTableFields)
    } else {
      // 没传入排序字段，则根据固定字段和自定义指标字段进行排序
      indicatorFields.forEach(item => {
        if (item.value && item.children) {
          item.children.forEach(child => {
            // 1. 使用对象合并设置默认值
            const defaults = {
              state: true,
              fixed: false,
              sort: false,
              type: TableFieldType.Default,
            }
            Object.assign(
              child,
              Object.fromEntries(
                Object.entries(defaults).filter(
                  ([k]) => (child as any)[k] === undefined,
                ),
              ),
            )

            // 2. 合并条件判断
            const shouldEnable = child.fixed || child.state
            if (shouldEnable) {
              item.value?.push(child.name)
              child.state = true // 确保状态同步
            }
            tableFieldsInfo.push(JSON.parse(JSON.stringify(child)))
          })
        } else {
          console.error('自定义指标字段没有value')
          // tableFieldsInfo.push({
          //   label: item.label,
          //   name: item.name,
          //   fixed: true,
          //   state: true,
          // } as any)
        }
      })
    }
  }

  /**
   * @description: 缓存表格数据。只有是远程查询的时候才会开启缓存
   *
   * @param isRemote 是否是远程查询
   * @param paginationConfig 分页配置
   * @param cacheTableData 缓存的表格数据
   * @param newData 新的表格数据
   */
  const setCacheTableData = (
    isRemote: boolean,
    paginationConfig: PaginationConfig,
    cacheTableData: Array<Array<TableData>>,
    newData: Array<TableData>,
  ) => {
    if (isRemote) {
      // 这页没有数据，就缓存一下
      const { curPage, curPageSize } = paginationConfig
      const pageIndex = curPage - 1
      if (
        !cacheTableData[pageIndex] ||
        cacheTableData[pageIndex].length === 0
      ) {
        cacheTableData[pageIndex] = newData.slice(0, curPageSize) // 缓存数据，后面的slice是防止返回的数据超出当前页长度
      }
    }
  }
  /**
   * @description: 导出表格数据位Excel未见
   * @param  propTableFields 表格字段信息
   * @param  tableData 表格数据
   * @param  fileName 保存的文件名
   * @param excludeExportFields 被排除的字段
   */
  const exportDataToExcel = (
    propTableFields: Array<BaseFieldItem<BaseFieldInfo>>,
    tableData: Array<any>,
    fileName: string,
    excludeExportFields: Array<string>,
  ) => {
    try {
      // 构建导出的表格数据
      const selectedData = tableData.slice(0, tableData.length)

      // 表头，这里重新生成一下
      // 这种形式：{name:'姓名', age:'年龄'}
      let header: {
        [key: string]: string
      } = {}
      propTableFields.forEach(item => {
        item.children?.forEach(child => {
          if (!excludeExportFields.includes(child.name))
            header[child.name] = child.label
        })
      })

      // 拼接数据
      const exportData = [header, ...selectedData]

      // 转为sheet对象，skipheader要选上，因为上面拼了一个表头
      const ws = utils.json_to_sheet(exportData, { skipHeader: true })

      // 创建工作部并且附加到一个工作表中，这个工作表名可以自定义
      const wb = utils.book_new()
      utils.book_append_sheet(wb, ws, fileName)
      // 导出
      writeFileXLSX(wb, fileName)
    } catch (err) {
      console.log(err)
      ElMessage.error('导出失败')
    }
  }

  return {
    tableReqList,
    getData,

    isFixedField,
    exportDataToExcel,
    updateTableFields,
    initTableFields,
    initIndicatorFields,
    setCacheTableData,
  }
}
