import type { UploadRawFile } from 'element-plus'

interface ImageProcessorConfig {
  timeout?: number // 超时时间（默认3000ms）
  crossOrigin?: string // 跨域模式（默认'anonymous'）
}

export class ImageProcessor {
  private image: HTMLImageElement
  private config: Required<ImageProcessorConfig>
  private objectUrl?: string

  constructor(
    source: string | File | Blob | UploadRawFile,
    config: ImageProcessorConfig = {},
  ) {
    this.config = {
      timeout: 3000,
      crossOrigin: 'anonymous',
      ...config,
    }

    // 创建图片元素
    this.image = new Image()
    this.image.crossOrigin = this.config.crossOrigin

    // 处理不同输入类型
    if (typeof source === 'string') {
      this.image.src = source
    } else {
      this.objectUrl = URL.createObjectURL(source)
      this.image.src = this.objectUrl
    }
  }

  /**
   * 获取图片分辨率
   */
  public async getResolution(): Promise<{ width: number; height: number }> {
    return new Promise((resolve, reject) => {
      // 超时处理
      const timeoutId = setTimeout(() => {
        this.cleanup()
        reject(new Error(`图片加载超时（${this.config.timeout}ms）`))
      }, this.config.timeout)

      // 成功加载
      const onLoad = () => {
        clearTimeout(timeoutId)
        this.cleanup()
        resolve({
          width: this.image.naturalWidth,
          height: this.image.naturalHeight,
        })
      }

      // 加载失败
      const onError = () => {
        clearTimeout(timeoutId)
        this.cleanup()
        reject(new Error('图片加载失败'))
      }

      this.image.addEventListener('load', onLoad, { once: true })
      this.image.addEventListener('error', onError, { once: true })
    })
  }

  /**
   * 清理资源
   */
  private cleanup() {
    if (this.objectUrl) {
      // URL.revokeObjectURL(this.objectUrl);
      this.objectUrl = undefined
    }
  }
}
