export interface VideoProcessorConfig {
  timeout?: number // 超时时间（默认10秒）
  crossOrigin?: string // 跨域设置（默认'anonymous'）
  coverQuality?: number // 封面质量 0-1（默认0.8）
  coverFormat?: 'jpeg' | 'png' // 封面格式（默认'jpeg'）
  fileName?: string // 新增文件名配置
}

export class VideoProcessor {
  private video: HTMLVideoElement
  private url: string
  private config: Required<VideoProcessorConfig>
  private cleanupHandlers: (() => void)[] = []

  // 新增初始化状态跟踪
  private isInitialized = false
  private initializationPromise: Promise<void> | null = null

  constructor(videoUrl: string, config: VideoProcessorConfig = {}) {
    this.url = videoUrl
    this.config = {
      timeout: 10000,
      crossOrigin: 'anonymous',
      coverQuality: 0.8,
      coverFormat: 'jpeg',
      fileName: '',
      ...config,
    }
    this.video = document.createElement('video')
  }

  /**
   * 初始化视频处理器
   */
  private async initialize(): Promise<void> {
    // 如果已经初始化完成，直接返回
    if (this.isInitialized) return

    // 如果初始化正在进行中，返回同一个Promise
    if (this.initializationPromise) {
      return this.initializationPromise
    }

    this.initializationPromise = new Promise((resolve, reject) => {
      // 配置基础属性
      this.video.crossOrigin = this.config.crossOrigin
      this.video.src = this.url

      const timeoutId = setTimeout(() => {
        this.cleanup()
        reject(new Error(`视频加载超时（${this.config.timeout}ms）`))
      }, this.config.timeout)

      const cleanup = () => {
        clearTimeout(timeoutId)
        this.removeAllEventListeners()
      }
      this.registerCleanup(cleanup)

      const onLoaded = () => {
        this.video.currentTime = Math.min(1, this.video.duration)
      }

      const onSeeked = () => {
        cleanup()
        this.isInitialized = true // 标记为已初始化
        resolve()
      }

      const onError = (err: ErrorEvent) => {
        cleanup()
        this.isInitialized = false // 初始化失败重置状态
        reject(err.error || new Error('视频加载失败'))
      }

      this.video.addEventListener('loadedmetadata', onLoaded)
      this.video.addEventListener('seeked', onSeeked)
      this.video.addEventListener('error', onError)
      this.registerCleanup(() => {
        this.video.removeEventListener('loadedmetadata', onLoaded)
        this.video.removeEventListener('seeked', onSeeked)
        this.video.removeEventListener('error', onError)
      })

      // 开始加载
      this.video.load()
    })
    return this.initializationPromise
  }

  /**
   * 获取视频基础信息
   */
  public async getVideoInfo() {
    await this.initialize()
    return {
      width: this.video.videoWidth,
      height: this.video.videoHeight,
      duration: this.video.duration,
      format: this.detectVideoFormat(),
    }
  }

  /**
   * 生成视频封面
   * @param timePoint 截取时间点（秒）
   */
  public async generateCover(timePoint = 1): Promise<string> {
    await this.initialize()

    return new Promise((resolve, reject) => {
      const targetTime = Math.min(timePoint, this.video.duration)

      const wrappedHandler = () => {
        try {
          this.video.removeEventListener('seeked', wrappedHandler)
          const canvas = document.createElement('canvas')
          const ctx = canvas.getContext('2d')

          if (!ctx) {
            throw new Error('无法获取Canvas上下文')
          }

          canvas.width = this.video.videoWidth
          canvas.height = this.video.videoHeight
          ctx.drawImage(this.video, 0, 0, canvas.width, canvas.height)

          const mimeType = `image/${this.config.coverFormat}`
          resolve(canvas.toDataURL(mimeType, this.config.coverQuality))
        } catch (err) {
          reject(err)
        }
      }

      this.video.addEventListener('seeked', wrappedHandler)
      this.video.currentTime = targetTime
    })
  }

  /**
   * 生成视频封面文件
   * @param timePoint 截取时间点（秒）
   */
  public async generateCoverFile(timePoint = 1): Promise<File> {
    await this.initialize()

    return new Promise((resolve, reject) => {
      const targetTime = Math.min(timePoint, this.video.duration)

      const wrappedHandler = () => {
        try {
          this.video.removeEventListener('seeked', wrappedHandler)
          const canvas = document.createElement('canvas')
          const ctx = canvas.getContext('2d')

          if (!ctx) {
            throw new Error('无法获取Canvas上下文')
          }

          canvas.width = this.video.videoWidth
          canvas.height = this.video.videoHeight
          ctx.drawImage(this.video, 0, 0, canvas.width, canvas.height)

          // 转换为Blob然后创建File对象
          canvas.toBlob(
            blob => {
              if (!blob) {
                reject(new Error('生成封面文件失败'))
                return
              }

              // 生成文件名
              const fileName = this.config.fileName || `cover_${Date.now()}`
              const fileExt = this.config.coverFormat === 'png' ? 'png' : 'jpg'
              const mimeType = `image/${this.config.coverFormat}`

              // 创建File对象
              const file = new File([blob], `${fileName}.${fileExt}`, {
                type: mimeType,
                lastModified: Date.now(),
              })

              resolve(file)
            },
            `image/${this.config.coverFormat}`,
            this.config.coverQuality,
          )
        } catch (err) {
          reject(err)
        }
      }

      this.video.addEventListener('seeked', wrappedHandler)
      this.video.currentTime = targetTime
    })
  }

  /**
   * 检测视频格式
   */
  private detectVideoFormat(): string {
    const matches = this.url.match(/\.(\w+)(\?|$)/)
    return matches ? matches[1].toLowerCase() : 'unknown'
  }

  /**
   * 注册清理函数
   */
  private registerCleanup(fn: () => void) {
    // 添加自检防止递归
    if (this.cleanupHandlers.includes(fn)) {
      console.warn('重复的清理函数已跳过')
      return
    }
    this.cleanupHandlers.push(fn)
  }
  /**
   * 移除所有事件监听
   */
  private removeAllEventListeners() {
    // 创建副本避免处理过程中数组变化
    const handlers = [...this.cleanupHandlers]
    this.cleanupHandlers = [] // 先清空数组

    // 反序执行防止依赖问题
    handlers.reverse().forEach(cleanup => {
      try {
        cleanup()
      } catch (err) {
        console.error('清理函数执行失败:', err)
      }
    })
  }

  /**
   * 资源清理
   */
  public cleanup() {
    this.removeAllEventListeners()
    this.video.pause()
    this.video.removeAttribute('src')
    this.video.load()

    // 安全释放ObjectURL
    if (this.url.startsWith('blob:')) {
      // URL.revokeObjectURL(this.url);
    }

    // 重置初始化状态
    this.isInitialized = false
    this.initializationPromise = null
  }
}
